#!/usr/local/bin/wish8.0
# This script will:
#  Load the mail extension
#  Build a small UI like a mail panel
#  Allow the user to put in a subject, body, and select addresses
#  Script is for Windows95 and WindowsNT: The addressee field needs to be
#  filled in differently for Win32s (an entry field, say).
# $Revision: 1.9 $
#

set addressees ""
set attachments 0

proc init_mapimail { } {
  if [ catch { package require mapimail } err ]  {
    if [ catch { load ./mapimail.dll } err ] {
      error "Can't load mapimail: $err"
    }
  }
}

proc do_address { } {
  global addressees
  if [ catch { mapimail address -clear } result ] {
    # skip
  } else {
    set addressees $result
  }
}

proc do_attach { } {
    global attachments
    set tmpatt $attachments
    set attachments ""
    set att [tk_getOpenFile -parent . -title "Select Attachment"]
    if { $att == "" } { 
        set attachments $tmpatt 
    } else {
        set attachments $att
    }
}

proc do_mail { } {
  global addressees subject attachments
  set text [ .frame.text.body get 1.0 end ]
  mapimail send -to $addressees -subject $subject -attachments $attachments -body $text
}

proc do_clear { } {
  global addressees
  set addressees ""
  global subject
  set subject ""
  .frame.text.body delete 1.0 end
}

proc do_send_comment { } {
  # Basically, just force everything into the right areas to prepare for sending a comment
  # Addressee: mschwart@nyx.net
  global addressees
  set addressees mschwart@nyx.net

  # Subject: Comment on mapimail
  global subject
  set subject "Comment on mapimail"
  
  # Attachment: none
  global attachments
  set attachments ""
  
  # Body: Include version number and platform information
  .frame.text.body delete 1.0 end
  lappend content "<Platform information. Please do not delete.>"
  lappend content "Mail version: [ mapimail version ]"

  global tcl_platform
  global tcl_patchLevel
  lappend content "Tcl Version: $tcl_patchLevel"
  lappend content "Tcl Platform: $tcl_platform(platform) $tcl_platform(os) $tcl_platform(osVersion) $tcl_platform(machine)\n"
  
  lappend content "\nMAPI subsystem details:\n"
  set enable  [ mapimail enabled ]
  set details [ lrange [ mapimail enabled -detail ] 1 end ]
  if { $enable == 1 } {
      lappend content "MAPI System Enabled\n"
  } else {
      lappend content "MAPI System Not Enabled: status $enable\n"
  }
  foreach detail $details {
      lappend content "[lindex $detail 0]:\t[lindex $detail 1]"
  }
  lappend content "\n"
  
  lappend content "<Add your own comments below, then click on 'Send' button>"
  lappend content "<Shouldn't every application have a Send Comment button?>\n"
  .frame.text.body insert end [join $content "\n"]
}

proc do_register { } {
  if [ catch { package require profile } err ] {
    if [ catch { load ./profile.dll } err ] {
      # "Null out" the profile package
      proc profile { args } { return "Never" }
    }
  }
  set sentmail [ profile get -private atk.ini -section Mail -key Sent -default Never ]

  if { "$sentmail" == "Never" } {
    set curdate [ clock format [clock seconds] ]

    if [ catch {
          mapimail send -to mschwart@nyx.net -subject "Running profile and mapimail!" \
          -body "Loaded at $curdate and running fine!"
             } err ] {
      error "Couldn't send mail: $err.\nPlease report any problems to the author."
    }

    profile set -private atk.ini -section Mail -key Sent -value $curdate

    tk_dialog .done "Registration Complete " \
            "Registration completed $sentmail. Thanks for using mapimail!" \
            info 0 OK

  } else {
    tk_dialog .done "Registration Complete" \
            "Registration skipped: previously completed $sentmail" \
	    info 0 OK
  }
}

proc change_mailer { new_mailer } {
  global mailer

  mapimail logoff
  mapimail option -mailer $new_mailer
  mapimail logon  
}

proc init_main_ui { } {
  global mailer

  if [ winfo exist .frame ] {
    destroy .frame
  }
  frame .frame

  # The menu part...
  menu  .frame.menu -tearoff 0
  menu  .frame.menu.file -tearoff 0
  .frame.menu add cascade -label File -menu .frame.menu.file -underline 0
  .frame.menu.file add command -label Exit -underline 0 -command { exit 0 }
  menu .frame.menu.options -tearoff 0
  .frame.menu add cascade -label Options -menu .frame.menu.options -underline 0
  .frame.menu.options add radio -label MAPI -variable mailer -value mapi \
				-command { change_mailer $mailer }
  .frame.menu.options add radio -label CMC  -variable mailer -value cmc  \
  				-command { change_mailer $mailer }
  .frame.menu.options add checkbutton -label Attachments -variable attachments \
  			           -indicatoron 1 -state normal 

  . configure -menu .frame.menu
  #  pack .frame.menu -side top -fill x -expand false
  set mailer mapi ;# Better .frame.menu.options invoke 1?

  # The work part
  frame .frame.text
  label .frame.text.subjectlabel -text Subject: -justify left
  entry  .frame.text.subject -textvariable subject
  label .frame.text.bodylabel -text Body: -justify left
  text  .frame.text.body
  label .frame.text.addresslabel -text Addressees: -justify left
  label .frame.text.addressees -textvariable addressees -wraplength 4i -justify left
  label .frame.text.attachlabel -text Attachment: -justify left
  label .frame.text.attachments -textvariable attachments -wraplength 4i -justify left
  frame .frame.buttons
  button .frame.buttons.ok -text Exit -command exit
  button .frame.buttons.address -text Address... -command do_address
  button .frame.buttons.attach  -text Attach...  -command do_attach
  button .frame.buttons.sendmail -text Send -command do_mail
  button .frame.buttons.sendcomment -text "Send Comment" -command do_send_comment
  button .frame.buttons.clearsend   -text "Clear" -command do_clear
  button .frame.buttons.readmail -text "In Box..." -command do_read_mail
  button .frame.buttons.about    -text About... -command about
  button .frame.buttons.register -text Register -command do_register

  grid configure .frame.text.subjectlabel -column 0 -row 0 -sticky nws
  grid configure .frame.text.subject -column 1 -row 0 -sticky news
  grid configure .frame.text.addresslabel -column 0 -row 1 -sticky nws
  grid configure .frame.text.addressees -column 1 -row 1 -sticky nws
  grid configure .frame.text.attachlabel -column 0 -row 2 -sticky nws
  grid configure .frame.text.attachments -column 1 -row 2 -sticky nws
  grid configure .frame.text.bodylabel -column 0 -row 3 -sticky w
  grid configure .frame.text.body -column 0 -columnspan 3 -row 4 -sticky news
  grid columnconfigure .frame.text 0 -weight 0
  grid columnconfigure .frame.text 1 -weight 2
  grid rowconfigure .frame.text 0 -weight 0
  grid rowconfigure .frame.text 1 -weight 0
  grid rowconfigure .frame.text 2 -weight 0
  grid rowconfigure .frame.text 3 -weight 0
  pack .frame.text -fill both -side right

  pack .frame.buttons.ok .frame.buttons.address .frame.buttons.attach .frame.buttons.sendmail \
       .frame.buttons.sendcomment .frame.buttons.clearsend \
       .frame.buttons.readmail .frame.buttons.register .frame.buttons.about \
       -side top -fill x
  pack .frame.buttons -side left -fill y

  pack .frame -side top -fill both

  # Titling part
  set tl [ winfo toplevel .frame ]
  wm title $tl "Mail Test for mapimail"
}

proc read_message { message } {
  global messagedata
  global attachments
  set id [ lindex $message 0 ]

  if { [info exist messagedata($id,Body)] == 0 } {
    set msg [mapimail read -id $messagedata($id,id) -attachments $attachments]
    foreach item $msg {
      if { [llength $item ] > 2 } {
        set messagedata($id,[lindex $item 0]) [lrange $item 1 end]
      } else {
        set messagedata($id,[lindex $item 0]) [lindex $item 1]
      }
    }
  }

  if [ winfo exist .message$id ] {
    .message$id.frame.message.text delete 1.0 end
    .message$id.frame.attach.list delete 0 end
    wm deiconify .message$id
    raise .message$id
  } else {
    toplevel .message$id
    wm title .message$id "Msg $id"
    wm iconname .message$id $id
    frame .message$id.frame

    frame .message$id.frame.message
    pack .message$id.frame.message -fill both -expand true
    scrollbar .message$id.frame.message.yscroll -command ".message$id.frame.message.text yview" -orient vertical
    text .message$id.frame.message.text \
		-yscroll ".message$id.frame.message.yscroll set"

    frame .message$id.frame.attach
    scrollbar .message$id.frame.attach.yscroll -orient vertical -command ".message$id.frame.attach.list yview"
    listbox .message$id.frame.attach.list -yscroll ".message$id.frame.attach.yscroll set" \
		-width 100 -height 3 -setgrid 1 -font { Courier 9 }

    pack .message$id.frame.attach.yscroll -side right -fill y -expand false
    pack .message$id.frame.attach.list -fill both -expand true

    pack .message$id.frame.message.yscroll -side right -fill y -expand false
    pack .message$id.frame.message.text -fill both -expand true

    pack .message$id.frame.message -side top -fill both -expand true
    pack .message$id.frame.attach  -side top -fill both -expand true

    pack .message$id.frame -fill both -expand true
  }
  if [ info exist messagedata($id,Body) ] {
    .message$id.frame.message.text insert end $messagedata($id,Body)
  } else {
    .message$id.frame.message.text insert end "<No Body>"
  }
  if [ info exist messagedata($id,Attachments:) ] {
    # First we have to either double backslashes or turn them into forward slashes...
    regsub -all {\\} $messagedata($id,Attachments:) / filenames
    foreach att $filenames {
      .message$id.frame.attach.list insert end $att
    }
  }
}

proc do_read_mail { } {
  global messagedata
  global attachments

  init_read_ui
  set counter 0
  mapimail logon
  set msg [ mapimail read -all -digest -reset -attachments $attachments ]
  while { $msg != "No Messages" } {
    foreach item $msg {
        if { [llength $item ] > 2 } {
          set messagedata($counter,[lindex $item 0]) [lrange $item 1 end]
        } else {
          set messagedata($counter,[lindex $item 0]) [lindex $item 1]
        }
    }
    if { [info exist messagedata($counter,Time)] == 0 } {
      set messagedata($counter,Time) "Unknown"
    }
    if { [info exist messagedata($counter,Originator)] == 0 } {
      set messagedata($counter,Originator) "Unknown"
    }
    if { [info exist messagedata($counter,Subject)] == 0 } {
      set messagedata($counter,Subject) "Unknown"
    }
    set string [format "%2d %-17.17s %-30.30s %-40.40s" $counter \
                $messagedata($counter,Time) $messagedata($counter,Originator) \
		$messagedata($counter,Subject)]
    .read.frame.list insert end $string
    incr counter
    set msg [ mapimail read -all -digest -attachments $attachments ]
  }
}

proc init_read_ui { } {
  if [ winfo exist .read ] {
    global messagedata
    destroy .read
  }
  if [ info exist messagedata ] {
    unset messagedata
  }
  toplevel .read
  wm title .read "Listing of Mail Inbox"
  wm iconname .read "In Box"

  frame .read.frame
  pack  .read.frame -fill both -expand true
  scrollbar .read.frame.scroll -command ".read.frame.list yview"
  listbox .read.frame.list   -yscroll ".read.frame.scroll set" \
                             -width 100 -height 15 -setgrid 1 -font { Courier 9}
  pack .read.frame.scroll -side right -fill y -expand false
  pack .read.frame.list -side left -fill both -expand true

  bind .read.frame.list <Double-1> {
    read_message [ selection get ]
  }  
}

proc about { } {
  if [ winfo exist .about ] {
    destroy .about
  }
  toplevel .about
  frame .about.text
  frame .about.buttons
  pack .about.buttons -side bottom -fill x -expand false
  pack .about.text -side top -fill both -expand true
  button .about.buttons.ok -text OK -command "destroy .about"
  pack .about.buttons.ok -expand false
  text .about.text.text
  pack .about.text.text -fill both -expand true
  .about.text.text insert 1.0 {
    This demonstration shows many of the features of the mail extension.
    The main panel explores how ANY Tcl application can be mail enabled.
    While the example uses Tk, none of the mail code requires a Tk UI to function.

    The "In Box" button shows some of the mail receipt functionality, though in a
    crude manner. This is due to my own bias that mail programs do their stuff well,
    and Tcl/Tk applications should interact well with them, not replace them.

    The first panel that comes up is a listing of the contents of the inbox.
    Double-clicking on any of these items will bring up a new toplevel with the
    body of the message in it.
    Attachments, recipients, and some other data is available as well.
    Both CMC and MAPI are supported, though the example uses MAPI.

    Please send suggestions, comments, good ideas you've implemented, and bug reports
    to the author:
            Michael I. Schwartz
            mschwart@nyx.net
            PO Box 24536
            Denver, CO 80224

    Special thanks to Chris Nelson (nelson@pinebush.com) for initial MSVC support,
    and Mark Austin (maustin@anadigics.com) for logon, logoff, read, and delete support.
  }
}

init_mapimail

init_main_ui
