/*
 -  SMAPI.C
 -
 *  Copyright 1993-1995 Microsoft Corporation. All Rights Reserved.
 */

#include <string.h>
#include <stdlib.h>
#include <windows.h>
#include <commdlg.h>
#include <mapiwin.h>
#include "smapi.h"
/*
#include <mapidbg.h>
#include "client.h"
#include "bitmap.h"
#include "pvalloc.h"
*/
HANDLE hInst;
HINSTANCE hlibMAPI = 0;

LPMAPILOGON lpfnMAPILogon = NULL;
LPMAPILOGOFF lpfnMAPILogoff = NULL;
LPMAPISENDMAIL lpfnMAPISendMail = NULL;
LPMAPISENDDOCUMENTS lpfnMAPISendDocuments = NULL;
LPMAPIFINDNEXT lpfnMAPIFindNext = NULL;
LPMAPIREADMAIL lpfnMAPIReadMail = NULL;
LPMAPISAVEMAIL lpfnMAPISaveMail = NULL;
LPMAPIDELETEMAIL lpfnMAPIDeleteMail = NULL;
LPMAPIFREEBUFFER lpfnMAPIFreeBuffer = NULL;
LPMAPIADDRESS lpfnMAPIAddress = NULL;
LPMAPIDETAILS lpfnMAPIDetails = NULL;
LPMAPIRESOLVENAME lpfnMAPIResolveName = NULL;

LPCMCFREE lpfncmc_free = NULL;
LPCMCSENDDOCUMENTS lpfncmc_send_documents = NULL;
LPCMCLOOKUP lpfncmc_look_up = NULL;
LPCMCLOGON  lpfncmc_logon = NULL;
LPCMCQUERYCONFIGURATION lpfncmc_query_configuration = NULL;
LPCMCLOGOFF lpfncmc_logoff = NULL;  
LPCMCREAD   lpfncmc_read = NULL;
LPCMCLIST   lpfncmc_list = NULL;
LPCMCACTON  lpfncmc_act_on = NULL;

/* Static Data */

static BOOL fDialogIsActive = FALSE;
static DWORD cUsers = 0;
static ULONG flSendMsgFlags = 0;
static LHANDLE lhSession = 0L;
static HBITMAP hReadBmp = 0;
static HBITMAP hReadABmp = 0;
static HBITMAP hUnReadBmp = 0;
static HBITMAP hUnReadABmp = 0;
static HCURSOR hWaitCur;
/* static LPMSGID lpReadMsgNode; */
static lpMapiMessage lpmsg = NULL;

#ifdef _WIN32
#define szMAPIDLL       "MAPI32.DLL"
#else
#define szMAPIDLL       "MAPI.DLL"
#endif

/*
 -  InitSimpleMAPI
 -
 *  Purpose:
 *      Loads the DLL containing the simple MAPI functions and sets
 *      up a pointer to each. Wrappers for the  function pointers
 *      are declared in SMAPI.H.
 *
 *  Returns:
 *      TRUE if sucessful, else FALSE
 *
 *  Side effects:
 *      Loads a DLL and sets up function pointers
 */
BOOL
InitSimpleMAPI (void)
{
    UINT fuError;

    fuError = SetErrorMode(SEM_NOOPENFILEERRORBOX);
    hlibMAPI = LoadLibrary(szMAPIDLL);
    SetErrorMode(fuError);

#ifdef _WIN32
    if (!hlibMAPI)
#else
    if (hlibMAPI < 32)
#endif
    return (FALSE);

    /* Changed strategy for smapi.c.
    ** Instead of failing altogether for missing functions, add another
    ** function to check what should be available.
    ** In this way, the proper behavior can be supported, whether that is
    ** to prevent the extension from working at all, limiting CMC, limiting
    ** to send only, etc.
    */
    lpfnMAPILogon = (LPMAPILOGON) GetProcAddress (hlibMAPI, "MAPILogon");
    lpfnMAPILogoff = (LPMAPILOGOFF) GetProcAddress (hlibMAPI, "MAPILogoff");
    lpfnMAPISendMail = (LPMAPISENDMAIL) GetProcAddress (hlibMAPI, "MAPISendMail");
    lpfnMAPISendDocuments = (LPMAPISENDDOCUMENTS) GetProcAddress (hlibMAPI, "MAPISendDocuments");
    lpfnMAPIFindNext = (LPMAPIFINDNEXT) GetProcAddress (hlibMAPI, "MAPIFindNext");
    lpfnMAPIReadMail = (LPMAPIREADMAIL) GetProcAddress (hlibMAPI, "MAPIReadMail");
    lpfnMAPISaveMail = (LPMAPISAVEMAIL) GetProcAddress (hlibMAPI, "MAPISaveMail");
    lpfnMAPIDeleteMail = (LPMAPIDELETEMAIL) GetProcAddress (hlibMAPI, "MAPIDeleteMail");
    lpfnMAPIFreeBuffer = (LPMAPIFREEBUFFER) GetProcAddress (hlibMAPI, "MAPIFreeBuffer");
    lpfnMAPIAddress = (LPMAPIADDRESS) GetProcAddress (hlibMAPI, "MAPIAddress");
    lpfnMAPIDetails = (LPMAPIDETAILS) GetProcAddress (hlibMAPI, "MAPIDetails");
    lpfnMAPIResolveName = (LPMAPIRESOLVENAME) GetProcAddress (hlibMAPI, "MAPIResolveName");

    lpfncmc_free = (LPCMCFREE) GetProcAddress (hlibMAPI, "cmc_free");
    lpfncmc_send_documents = (LPCMCSENDDOCUMENTS) GetProcAddress (hlibMAPI, "cmc_send_documents");
    lpfncmc_look_up = (LPCMCLOOKUP) GetProcAddress (hlibMAPI, "cmc_look_up");
    lpfncmc_logon = (LPCMCLOGON) GetProcAddress (hlibMAPI, "cmc_logon");
    lpfncmc_query_configuration = (LPCMCQUERYCONFIGURATION) GetProcAddress (hlibMAPI, "cmc_query_configuration");
    lpfncmc_logoff = (LPCMCLOGOFF) GetProcAddress (hlibMAPI, "cmc_logoff");
    lpfncmc_read = (LPCMCREAD) GetProcAddress (hlibMAPI, "cmc_read");
    lpfncmc_list = (LPCMCLIST) GetProcAddress (hlibMAPI, "cmc_list");
    lpfncmc_act_on = (LPCMCACTON) GetProcAddress (hlibMAPI, "cmc_act_on");

    return (TRUE);
}

void
DeinitSimpleMAPI ()
{
  if (hlibMAPI)
  {
    FreeLibrary (hlibMAPI);
    hlibMAPI = 0;
  }
}



